<?php

namespace Drupal\suggest_similar_titles\Controller;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

class SuggestionController extends ControllerBase {

  /**
   * Database connection
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $connection;

  /**
   * The current request.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * The module config.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $config;

  /**
   * Renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * The controller constructor.
   *
   * @param \Drupal\Core\Database\Connection $connection
   *   Database connection.
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The current request.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   Renderer service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   Config factory service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity type manager service.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   Language manager.
   */
  public function __construct(Connection $connection, Request $request, RendererInterface $renderer, ConfigFactoryInterface $config_factory, EntityTypeManagerInterface $entity_type_manager, LanguageManagerInterface $language_manager) {
    $this->connection = $connection;
    $this->request = $request;
    $this->configFactory = $config_factory;
    $this->config = $config_factory->get('suggest_similar_titles.settings');
    $this->renderer = $renderer;
    $this->entityTypeManager = $entity_type_manager;
    $this->languageManager = $language_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database'),
      $container->get('request_stack')->getCurrentRequest(),
      $container->get('renderer'),
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
      $container->get('language_manager')
    );
  }

  /**
   * Display the markup of the suggestions.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   A response object serving node suggestions.
   */
  public function suggest_similar_titles_ajax_title() {
    $node_title = trim($this->request->get('title'));
    $title_processed = $this->processTitle($node_title);
    $node_type = trim($this->request->get('type'));
    $data = '';

    if (!empty($title_processed)) {

      $matches = $this->getMatchingTitles($title_processed, $node_type);
      $node_array = $this->processMatches($title_processed, $matches);

      if ($node_array) {
        // Calling theme function and passing nodes array.
        $data = [
				  '#theme' => 'suggest_similar_titles',
				  '#nodes' => $node_array,
        ];
      }
    }

    // Returning data to ajax call.
    $response = new Response();
    $response->setContent($this->renderer->renderRoot($data));

    return $response;
  }

  /**
   * Get matching node titles from database.
   *
   * @param array $title_parts
   *   Array containing all parts of the node title to match to other nodes.
   * @param string $type
   *   The node type to compare titles to.
   *
   * @return array
   *   Array containing objects with 2 properties:
   *     - nid
   *     - title
   */
  protected function getMatchingTitles($title_parts, $type = NULL) {
    $query = $this->connection->select('node', 'n');
    $query->join('node_field_data', 'nfd', 'n.nid = nfd.nid');
    $query->fields('n', ['nid']);
    $query->fields('nfd', ['title']);
    $or_group = $query->orConditionGroup();

    foreach ($title_parts as $value) {
      // Case insensitive LIKE query.
      $or_group->condition('nfd.title', '%' . $this->connection->escapeLike($value) . '%', 'LIKE');
    }

    $query->condition($or_group);
    if ($type && $this->config->get('limit_matching_to_type')) {
      $query->condition('n.type', $type);
    }

    return $query->execute()->fetchAll();
  }

  /**
   * Process the matched node titles.
   *
   * @param array $title_parts
   *   Original title coming from new node, split into an array.
   * @param array $matches
   *   Matching node titles as objects coming from getMatchingTitles().
   *
   * @return array
   *   Array containing the sub arrays with keys:
   *   - node
   *   - link
   *   - type_label
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   *   Thrown if the entity type doesn't exist.
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   *   Thrown if the storage handler couldn't be loaded.
   */
  protected function processMatches($title_parts, $matches) {
    $node_array = [];
    $number_of_nodes = $this->config->get('number_of_nodes');
    $check_permissions = $this->config->get('node_access');
    $percentage = $this->config->get('match_percentage');
    $display_type = $this->config->get('display_type');
    $desired_ratio = 100 / $percentage;

    // Set config language to retrieve correct translations.
    $language = $this->languageManager->getCurrentLanguage();
    $original_language = $this->languageManager->getConfigOverrideLanguage();
    $this->languageManager->setConfigOverrideLanguage($language);

    foreach ($matches as $match) {
      $matched_title_parts = $this->processTitle($match->title);
      $count = count($matched_title_parts) >= count($title_parts) ? count($title_parts) : count($matched_title_parts);
      $words_match = count(array_intersect($title_parts, $matched_title_parts));

      if ($words_match > 0) {
        $ratio = $count / $words_match;

        if ($ratio <= $desired_ratio) {
          $node = $this->entityTypeManager->getStorage('node')->load($match->nid);

          if (!$check_permissions || ($check_permissions && $node->access('view'))) {
            $node_type = $this->entityTypeManager->getStorage('node_type')->load($node->bundle());
            $node_array[$match->nid] = [
              'node' => $node,
              'link' => Url::fromUri('internal:/node/' . $match->nid, ['attributes' => ['target' => '_blank']]),
              'type_label' => $display_type ? $node_type->label() : '',
            ];
          }
        }
      }

      if (count($node_array) === (int) $number_of_nodes) {
        break;
      }
    }

    // Return config language to original state.
    $this->languageManager->setConfigOverrideLanguage($original_language);

    return $node_array;
  }

  /**
   * Process and filter title based on configuration.
   *
   * @param string $title
   *   A node title.
   *
   * @return array
   *   The processed title and split up to array.
   */
  protected function processTitle($title) {
    $title = strtolower($title);
    $ignored = $this->config->get('ignored');
    $ignored_array = explode(',', $ignored);
    $title_array = explode(' ', $title);

    return array_diff($title_array, $ignored_array);
  }

}
