<?php

namespace Drupal\suggest_similar_titles\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Form\FormStateInterface;

/**
 * Defines a form that configures forms module settings.
 */
class SuggestionConfigurationForm extends ConfigFormBase {

  /**
   * The footer settings configuration.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $suggestionConfig;

  /**
   * Entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs the form object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity type manager service
   */
  public function __construct(ConfigFactoryInterface $config_factory, EntityTypeManagerInterface $entity_type_manager) {
    parent::__construct($config_factory);

    $this->suggestionConfig = $this->config('suggest_similar_titles.settings');
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'suggest_similar_titles_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'suggest_similar_titles.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, Request $request = NULL) {
    $types = $this->entityTypeManager->getStorage('node_type')->loadMultiple();

    $form['fieldset_suggest_title'] = [
      '#type' => 'fieldset',
      '#title' => t('Content types'),
      '#description' => t('Select content type to enable similar titles suggestion for.'),
    ];

    foreach ($types as $key => $type) {
      $form['fieldset_suggest_title']['enable_type_' . $key] = [
        '#type' => 'checkbox',
        '#title' => $type->label(),
        '#default_value' => NULL !== $this->suggestionConfig->get('enable_type_' . $key) ? $this->suggestionConfig->get('enable_type_' . $key) : NULL,
      ];
    }

    $form['fieldset_suggest_title_display_settings'] = [
      '#type' => 'fieldset',
      '#title' => t('Display settings'),
    ];

    $form['fieldset_suggest_title_display_settings']['position'] = [
      '#type' => 'select',
      '#title' => t('Position'),
      '#options' => ['top' => 'Top', 'bottom' => 'Bottom'],
      '#default_value' => NULL !== $this->suggestionConfig->get('position') ? $this->suggestionConfig->get('position') : 'top',
      '#description' => t('Select position where you want to show similar titles suggestion list.'),
    ];

    $form['fieldset_suggest_title_display_settings']['number_of_nodes'] = [
      '#type' => 'select',
      '#title' => t('Number of nodes'),
      '#options' => [1 => '1', 5 => '5', 10 => '10', 20 => '20'],
      '#default_value' => NULL !== $this->suggestionConfig->get('number_of_nodes') ? $this->suggestionConfig->get('number_of_nodes') : 5,
      '#description' => t('Select maximum number of nodes to display as similar titles.'),
    ];

    $form['fieldset_suggest_title_display_settings']['display_type'] = [
      '#type' => 'checkbox',
      '#title' => t('Display content type'),
      '#description' => t('If checked, the content type will be displayed in the suggestion list.'),
      '#default_value' => NULL !== $this->suggestionConfig->get('display_type') ? $this->suggestionConfig->get('display_type') : 1,
    ];

    $form['fieldset_suggest_title_display_settings']['node_access'] = [
      '#type' => 'checkbox',
      '#title' => t('Consider node permissions'),
      '#default_value' => NULL !== $this->suggestionConfig->get('node_access') ? $this->suggestionConfig->get('node_access') : 0,
      '#description' => t('Select whether the system should check node view permissions before displaying the node in the similar titles suggestion list.<br />
			If checked, the system will not display restricted nodes to the user.'),
    ];

    $form['fieldset_suggest_title_compare_settings'] = [
      '#type' => 'fieldset',
      '#title' => t('Title compare patterns'),
    ];

    $form['fieldset_suggest_title_compare_settings']['limit_matching_to_type'] = [
      '#type' => 'checkbox',
      '#title' => t('Limit title matching to same content type'),
      '#description' => t('If checked, only titles of the same content type will we searched for.'),
      '#default_value' => NULL !== $this->suggestionConfig->get('limit_matching_to_type') ? $this->suggestionConfig->get('limit_matching_to_type') : 0,
    ];

    $form['fieldset_suggest_title_compare_settings']['ignored'] = [
      '#type' => 'textfield',
      '#title' => t('Ignore keywords'),
      '#description' => t('Enter comma separated keywords to ignore in title comparison.'),
      '#default_value' => NULL !== $this->suggestionConfig->get('ignored') ? $this->suggestionConfig->get('ignored') : 'the,is,a',
    ];

    $form['fieldset_suggest_title_compare_settings']['match_percentage'] = [
      '#type' => 'textfield',
      '#title' => t('Percentage'),
      '#description' => t('Enter percentage how exact system should compare the title. For example, if you enter 75, then an at least 75% matching title will be considered similar.'),
      '#default_value' => NULL !== $this->suggestionConfig->get('match_percentage') ? $this->suggestionConfig->get('match_percentage') : 75,
      '#size' => 4,
      '#maxlength' => 2,
      '#field_suffix' => '%',
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);

    $values = $form_state->getValues();

    $this->suggestionConfig->set('position', $values['position']);
    $this->suggestionConfig->set('number_of_nodes', $values['number_of_nodes']);
    $this->suggestionConfig->set('node_access', $values['node_access']);
    $this->suggestionConfig->set('limit_matching_to_type', $values['limit_matching_to_type']);
    $this->suggestionConfig->set('display_type', $values['display_type']);
    $this->suggestionConfig->set('ignored', $values['ignored']);
    $this->suggestionConfig->set('match_percentage', $values['match_percentage']);

    $types = $this->entityTypeManager->getStorage('node_type')->loadMultiple();

    foreach ($types as $key => $type) {
      $this->suggestionConfig->set('enable_type_' . $key, $values['enable_type_' . $key]);
    }

    $this->suggestionConfig->save();
  }

}
